@@ -4,5 +4,5 @@ import logging
 logger = logging.getLogger(__name__)
 debug_conversion = False
 
-# logger.setLevel(logging.DEBUG)
+logger.setLevel(logging.DEBUG)
 #debug_conversion = True
