@@ -111,7 +111,7 @@ def test_simple_circular_binop_promotion():
 def test_circular_binop():
     """
     >>> test_circular_binop()
-    (1.0, 2.0, 1.0, -3L)
+    (1.0, 2.0, 1.0, -3)
     >>> sig, syms = infer(test_circular_binop.py_func,
     ...                   functype(None, []), warn=False)
     >>> types(syms, 'x', 'y', 'z', 'a')
@@ -176,7 +176,7 @@ def test_circular_compare3():
     2
     3
     4
-    (False, 10L)
+    (False, 10)
     >>> sig, syms = infer(test_circular_compare3.py_func,
     ...                   functype(None, []), warn=False)
     >>> types(syms, 'cond')
@@ -208,7 +208,7 @@ def test_circular_compare3():
 def test_delayed_array_indexing():
     """
     >>> test_delayed_array_indexing()
-    (array([ 0.,  1.,  2.,  3.,  4.,  5.,  6.,  7.,  8.,  9.]), 1.0, 10L)
+    (array([ 0.,  1.,  2.,  3.,  4.,  5.,  6.,  7.,  8.,  9.]), 1.0, 10)
     >>> sig, syms = infer(test_delayed_array_indexing.py_func,
     ...                   functype(None, []), warn=False)
     >>> types(syms, 'array', 'var', 'x')
@@ -272,7 +272,7 @@ def test_delayed_array_slicing2():
 def test_delayed_string_indexing_simple():
     """
     >>> test_delayed_string_indexing_simple()
-    ('eggs', 3L)
+    ('eggs', 3)
     >>> sig, syms = infer(test_delayed_string_indexing_simple.py_func,
     ...                   functype(None, []), warn=False)
     >>> types(syms, 's', 'x')
@@ -292,7 +292,7 @@ def test_delayed_string_indexing_simple():
 def test_delayed_string_indexing():
     """
     >>> test_delayed_string_indexing()
-    ('ham eggs', 3L)
+    ('ham eggs', 3)
     >>> sig, syms = infer(test_delayed_string_indexing.py_func,
     ...                   functype(None, []), warn=False)
     >>> types(syms, 's', 'x')
@@ -318,7 +318,7 @@ def test_delayed_string_indexing():
 def test_delayed_string_indexing2():
     """
     >>> test_delayed_string_indexing2()
-    ('ham eggs', 3L)
+    ('ham eggs', 3)
     >>> sig, syms = infer(test_delayed_string_indexing2.py_func,
     ...                   functype(None, []), warn=False)
     >>> types(syms, 's', 'x')
@@ -375,8 +375,8 @@ def test_string_indexing_error2():
 @autojit(warn=False, warnstyle='simple')
 def test_string_indexing_valid():
     """
-    >>> chr(test_string_indexing_valid())
-    'm'
+    >>> test_string_indexing_valid() == b'm'
+    True
     """
     for i in range(4):
         s = "spam ham eggs"
@@ -397,7 +397,7 @@ def simple_func(x):
 def test_simple_call():
     """
     >>> test_simple_call()
-    1091100052L
+    1091100052
     >>> infer_simple(test_simple_call, 'x')
     (int,)
     """
@@ -452,6 +452,27 @@ def test_simple_call_promotion2():
 
 #print test_simple_call_promotion2.py_func()
 
+#------------------------------------------------------------------------
+# Delayed Attributes
+#------------------------------------------------------------------------
+
+@autojit(warn=False)
+def test_delayed_attributes1(A):
+    """
+    >>> A = np.empty(2, dtype=[('a', np.int32), ('b', np.float64)])
+    >>> list(test_delayed_attributes1(A))
+    [(1, 2.0), (2, 4.0)]
+    """
+    idx = 0
+    for i in range(A.shape[0]):
+        A[idx].a = i + 1
+        A[idx].b = A[idx].a * 2
+        idx += 1
+        if idx > 5:
+            idx = 5
+
+    return A
+
 #------------------------------------------------------------------------
 # Test Utilities
 #------------------------------------------------------------------------
