@@ -1,22 +1,26 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function, division, absolute_import
+
+import os
 import weakref
 import ast as ast_module
 import types
 import logging
 import pprint
+import collections
 
 import llvm.core
 
 from numba import pipeline, naming, error, reporting, PY3
-from numba.control_flow.control_flow import ControlFlow
 from numba.utils import TypedProperty, WriteOnceTypedProperty, NumbaContext
 from numba import functions, symtab
 from numba.typesystem import TypeSystem, numba_typesystem, function
 from numba.utility.cbuilder import library
 from numba.nodes import metadata
+from numba.control_flow import ControlFlow
 from numba.codegen import translate
 from numba.codegen import globalconstants
+from numba.ndarray_helpers import NumpyArray
 
 from numba.intrinsic import default_intrinsic_library
 from numba.external import default_external_library
@@ -34,20 +38,34 @@ else:
     NoneType = types.NoneType
     name_types = (str, unicode)
 
-default_pipeline_order = [
+default_normalize_order = [
     'ast3to2',
     'resolve_templates',
     'validate_signature',
     'update_signature',
     'create_lfunc1',
     'NormalizeASTStage',
+    'TransformBuiltinLoops',
+    'ValidateASTStage',
+]
+
+default_cf_pipeline_order = ['ast3to2', 'ControlFlowAnalysis']
+
+
+default_pipeline_order = default_normalize_order + [
+    'ExpandPrange',
+    'RewritePrangePrivates',
+    'FixASTLocations',
     'ControlFlowAnalysis',
+    'dump_cfg',
     #'ConstFolding',
+    # 'dump_ast',
+    'UpdateAttributeStatements',
     'TypeInfer',
+    'CleanupPrange',
     'update_signature',
     'create_lfunc2',
     'TypeSet',
-    'dump_cfg',
     'ClosureTypeInference',
     'create_lfunc3',
     'TransformFor',
@@ -57,27 +75,27 @@ default_pipeline_order = [
     'SpecializeSSA',
     'SpecializeClosures',
     'Optimize',
-    'Preloader',
     'SpecializeLoops',
+    'LowerRaise',
     'FixASTLocations',
     'LateSpecializer',
     'ExtensionTypeLowerer',
     'SpecializeFunccalls',
     'SpecializeExceptions',
-    'FixASTLocations',
     'cleanup_symtab',
+    'validate_arrays',
+    'dump_ast',
+    'FixASTLocations',
     'CodeGen',
+    'dump_annotations',
+    'dump_llvm',
     'PostPass',
     'LinkingStage',
+    'dump_optimized',
     'WrapperStage',
     'ErrorReporting',
 ]
 
-default_type_infer_pipeline_order = [
-    'ast3to2',
-    'ControlFlowAnalysis',
-    'TypeInfer',
-]
 
 default_dummy_type_infer_pipeline_order = [
     'ast3to2',
@@ -103,6 +121,7 @@ default_numba_late_translate_pipeline_order = \
 upto = lambda order, x: order[:order.index(x)+1]
 upfr = lambda order, x: order[order.index(x)+1:]
 
+default_type_infer_pipeline_order = upto(default_pipeline_order, 'TypeInfer')
 default_compile_pipeline_order = upfr(default_pipeline_order, 'TypeInfer')
 default_codegen_pipeline = upto(default_pipeline_order, 'CodeGen')
 default_post_codegen_pipeline = upfr(default_pipeline_order, 'CodeGen')
@@ -207,6 +226,7 @@ class FunctionEnvironment(object):
         False)
 
     func_name = TypedProperty(str, 'Target function name.')
+    module_name = TypedProperty(str, 'Name of the function module.')
 
     mangled_name = TypedProperty(str, 'Mangled name of compiled function.')
 
@@ -279,11 +299,13 @@ class FunctionEnvironment(object):
         'numba.typesystem.templatetypes.')
 
     typesystem = TypedProperty(TypeSystem, "Typesystem for this compilation")
+    array = TypedProperty(object, "Array abstraction", NumpyArray)
 
     ast_metadata = TypedProperty(
         object,
         'Metadata for AST nodes of the function being compiled.')
 
+    warn = True
     flow = TypedProperty(
         (NoneType, ControlFlow),
         "Control flow graph. See numba.control_flow.",
@@ -295,6 +317,26 @@ class FunctionEnvironment(object):
         object, # Should be ControlFlowAnalysis.
         'The Control Flow Analysis transform object '
         '(control_flow.ControlFlowAnalysis). Set during the cfg pass.')
+    cfdirectives = TypedProperty(
+        dict, "Directives for control flow.",
+        default={
+            'warn.maybe_uninitialized': warn,
+            'warn.unused_result': False,
+            'warn.unused': warn,
+            'warn.unused_arg': warn,
+            # Set the below flag to a path to generate CFG dot files
+            'control_flow.dot_output': os.path.expanduser("~/cfg.dot"),
+            'control_flow.dot_annotate_defs': False,
+        },
+    )
+
+    kill_attribute_assignments = TypedProperty( # Prange
+        (set, frozenset),
+        "Assignments to attributes that need to be removed from type "
+        "inference pre-analysis. We need to do this for prange since we "
+        "need to infer the types of variables to build a struct type for "
+        "those variables. So we need type inference to be properly ordered, "
+        "and not look at the attributes first.")
 
     # FIXME: Get rid of this property; pipeline stages are users and
     # transformers of the environment.  Any state needed beyond a
@@ -335,6 +377,14 @@ class FunctionEnvironment(object):
         'Flag that enables control flow warnings on a per-function level.',
         True)
 
+    annotations = TypedProperty(
+        dict, "Annotation dict { lineno : Annotation }"
+    )
+
+    intermediates = TypedProperty(
+        list, "list of Intermediate objects for annotation",
+    )
+
     warnstyle = TypedProperty(
         str if PY3 else basestring,
         'Warning style, currently available: simple, fancy',
@@ -363,11 +413,11 @@ class FunctionEnvironment(object):
              llvm_module=None, wrap=True, link=True,
              symtab=None,
              error_env=None, function_globals=None, locals=None,
-             template_signature=None, cfg_transform=None,
-             is_closure=False, closures=None, closure_scope=None,
+             template_signature=None, is_closure=False,
+             closures=None, closure_scope=None,
              refcount_args=True,
              ast_metadata=None, warn=True, warnstyle='fancy',
-             typesystem=None, postpasses=None,
+             typesystem=None, array=None, postpasses=None, annotate=False,
              **kws):
 
         self.parent = parent
@@ -387,6 +437,16 @@ class FunctionEnvironment(object):
         else:
             qname = name
 
+        if function_globals is not None:
+            self.function_globals = function_globals
+        else:
+            self.function_globals = self.func.__globals__
+
+        if self.func:
+            self.module_name = self.func.__module__
+        else:
+            self.module_name = self.function_globals.get("__name__", "")
+
         if mangled_name is None:
             mangled_name = naming.specialized_mangle(qname,
                                                      self.func_signature.args)
@@ -396,6 +456,8 @@ class FunctionEnvironment(object):
         self.qualified_name = qualified_name or name
         self.llvm_module = (llvm_module if llvm_module
                                  else self.numba.llvm_context.module)
+
+        self._annotate = annotate
         self.wrap = wrap
         self.link = link
         self.llvm_wrapper_func = None
@@ -405,20 +467,19 @@ class FunctionEnvironment(object):
                                                                self.ast,
                                                                warnstyle)
 
-        if function_globals is not None:
-            self.function_globals = function_globals
-        else:
-            self.function_globals = self.func.__globals__
 
         self.locals = locals if locals is not None else {}
         self.template_signature = template_signature
-        self.cfg_transform = cfg_transform
         self.is_closure = is_closure
         self.closures = closures if closures is not None else {}
         self.closure_scope = closure_scope
+        self.kill_attribute_assignments = set()
 
         self.refcount_args = refcount_args
         self.typesystem = typesystem or numba_typesystem
+        if array:
+            self.array = array
+            # assert issubclass(array, NumpyArray)
 
         import numba.postpasses
         self.postpasses = postpasses or numba.postpasses.default_postpasses
@@ -428,6 +489,8 @@ class FunctionEnvironment(object):
         else:
             self.ast_metadata = metadata.create_metadata_env()
 
+        self.annotations = collections.defaultdict(list)
+        self.intermediates = []
         self.warn = warn
         self.warnstyle = warnstyle
         self.kwargs = kws
@@ -448,9 +511,9 @@ class FunctionEnvironment(object):
             function_globals=self.function_globals,
             locals=self.locals,
             template_signature=self.template_signature,
-            cfg_transform=self.cfg_transform,
             is_closure=self.is_closure,
             closures=self.closures,
+            kill_attribute_assignments=self.kill_attribute_assignments,
             closure_scope=self.closure_scope,
             warn=self.warn,
             warnstyle=self.warnstyle,
@@ -468,6 +531,11 @@ class FunctionEnvironment(object):
         state.update(kwds)
         return type(self)(**state)
 
+    @property
+    def annotate(self):
+        "Whether we need to annotate the source"
+        return self._annotate or self.numba.cmdopts.get('annotate')
+
     @property
     def func_doc(self):
         if self.func is not None:
@@ -743,6 +811,14 @@ class NumbaEnvironment(_AbstractNumbaEnvironment):
         default=globalconstants.LLVMConstantsManager(),
     )
 
+    cmdopts = TypedProperty(
+        dict, "Dict of command line options from bin/numba.py", {},
+    )
+
+    annotation_blocks = TypedProperty(
+        list, "List of annotation information for different functions."
+    )
+
     # ____________________________________________________________
     # Class members
 
@@ -761,6 +837,10 @@ class NumbaEnvironment(_AbstractNumbaEnvironment):
             default_pipeline_order)
         self.pipelines = {
             self.default_pipeline : actual_default_pipeline,
+            'normalize' : pipeline.ComposedPipelineStage(
+                default_normalize_order),
+            'cf' : pipeline.ComposedPipelineStage(
+                default_cf_pipeline_order),
             'type_infer' : pipeline.ComposedPipelineStage(
                 default_type_infer_pipeline_order),
             'dummy_type_infer' : pipeline.ComposedPipelineStage(
@@ -796,6 +876,7 @@ class NumbaEnvironment(_AbstractNumbaEnvironment):
         context.external_library = default_external_library(context)
         context.utility_library = default_utility_library(context)
         self.llvm_context = translate.LLVMContextManager()
+        self.annotation_blocks = []
 
     def link_cbuilder_utilities(self):
         self.context.cbuilder_library = library.CBuilderLibrary()
