@@ -42,59 +42,63 @@ True
 
 from numba import *
 
-@jit(backend='ast')
+### Python usage
+
+@jit(object_())
 def abs_as_name():
     x = abs
     return x
 
-@autojit(backend='ast')
+@jit(argtypes=[object_])
+def py_abs(a):
+    return abs(a)
+
+### nopython usage
+
+@autojit(nopython=True)
 def _abs(value):
     result = abs(value)
     with nopython:
         return result * 2 # test return type being non-object
 
-@jit(backend='ast', argtypes=[object_])
-def py_abs(a):
-    return abs(a)
-
-@jit(backend='ast', argtypes=[int_])
+@jit(nopython=True, argtypes=[int_])
 def int_abs(a):
     return _abs(a)
 
-@jit(backend='ast', argtypes=[long_])
+@jit(nopython=True, argtypes=[long_])
 def long_abs(a):
     return _abs(a)
 
-@jit(backend='ast', argtypes=[ulong])
+@jit(nopython=True, argtypes=[ulong])
 def ulong_abs(a):
     return _abs(a)
 
-@jit(backend='ast', argtypes=[int32])
+@jit(nopython=True, argtypes=[int32])
 def int32_abs(a):
     return _abs(a)
 
-@jit(backend='ast', argtypes=[longlong])
+@jit(nopython=True, argtypes=[longlong])
 def long_long_abs(a):
     return _abs(a)
 
-@jit(backend='ast', argtypes=[ulonglong])
+@jit(nopython=True, argtypes=[ulonglong])
 def ulong_long_abs(a):
     return _abs(a)
 
-@jit(backend='ast', argtypes=[double])
+@jit(nopython=True, argtypes=[double])
 def double_abs(a):
     return _abs(a)
 
-@jit(backend='ast', argtypes=[float_])
+@jit(nopython=True, argtypes=[float_])
 def float_abs(a):
     return _abs(a)
 
-@jit(backend='ast', argtypes=[complex64])
+@jit(nopython=True, argtypes=[complex64])
 def complex64_abs(a):
     return _abs(a)
 
 
-@jit(backend='ast', argtypes=[complex128])
+@jit(nopython=True, argtypes=[complex128])
 def complex128_abs(a):
     return _abs(a)
 
